"use strict";

var _big = _interopRequireDefault(require("big.js"));

var _superagent = _interopRequireDefault(require("superagent"));

var _memoize = _interopRequireDefault(require("./memoize"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

require('superagent-proxy')(_superagent["default"]);

var enc = encodeURIComponent; // if none is set, will fallback to using HTTPS_PROXY or ALL_PROXY.
// see https://github.com/Rob--W/proxy-from-env

var RATE_PROXY = process.env.RATE_PROXY;
var FIXED_RATES = {
  BTC: 1
},
    BTC_MSAT_RATIO = (0, _big["default"])('100000000000'),
    BITSTAMP_CURR = ['USD', 'EUR'],
    CACHE_TTL = +process.env.RATE_CACHE_TTL || 30000; // Fetch USD/EUR rates from Bitstamp, use CoinGecko for other currencies

var getRate = (0, _memoize["default"])(CACHE_TTL, function (currency) {
  return BITSTAMP_CURR.includes(currency.toUpperCase()) ? getRateBitstamp(currency) : getRateCoingecko(currency);
});

var getRateBitstamp = function getRateBitstamp(currency) {
  return _superagent["default"].get("https://www.bitstamp.net/api/v2/ticker/btc".concat(currency.toLowerCase())).proxy(RATE_PROXY).then(function (res) {
    return res.body.last || Promise.reject('Invalid response from Bitstamp');
  });
};

var getRateCoingecko = function getRateCoingecko(currency) {
  return _superagent["default"].get("https://api.coingecko.com/api/v3/simple/price?ids=bitcoin&vs_currencies=".concat(enc(currency))).proxy(RATE_PROXY).then(function (res) {
    return res.body.bitcoin[currency.toLowerCase()] || Promise.reject("Unknown currency: ".concat(currency));
  });
}; // Convert `amount` units of `currency` to msatoshis


var toMsat = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(currency, amount) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.t0 = (0, _big["default"])(amount);
            _context.t1 = FIXED_RATES[currency];

            if (_context.t1) {
              _context.next = 6;
              break;
            }

            _context.next = 5;
            return getRate(currency);

          case 5:
            _context.t1 = _context.sent;

          case 6:
            _context.t2 = _context.t1;
            return _context.abrupt("return", _context.t0.div.call(_context.t0, _context.t2).mul(BTC_MSAT_RATIO).round(0, 3). // round up to nearest msatoshi
            toFixed(0));

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function toMsat(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

module.exports = {
  getRate: getRate,
  toMsat: toMsat
};